
/*================================================*\

	Mardi 24 novembre 1998

	Arash HABIBI

	Attente.c

	A chaque fois qu'il faut attendre un 
	evenement c'est ici qu'il faut s'adresser. 

\*================================================*/


#include "Attente.h"


int Max(int x, int y, int z)
/*================================================*\
  Rend le plus grand des trois entiers. 
\*================================================*/
{
  if(x>y)
  {     if(x>z)      return(x);
	else /*z>x*/ return(z);
  }
  else
  {     if(y>z)      return(y);
	else /*z>y*/ return(z);
  }
}


int Attendre(int fd1, int fd2, int fd3, float timeout)
/*================================================*\
  Attend que l'un des deux descripteurs fd1, fd2 ou
  fd3 ou les trois deviennent lisibles ou alors que 
  le temps "timeout" (en secondes) soit ecoule. fd1 
  fd2 et fd3 peuvent etre des descripteurs de fichier, 
  de terminal, de socket ou de tube. 
  
  Si timeout=0 alors le temporisateur ne sera pas 	 
  arme. De meme si l'un des descripteurs est negatif
  (par exemple RIEN) alors l'argument sera ignore. 
  En particulier Attendre(RIEN, RIEN, 3.0); ne fait 
  qu'attendre 3 secondes avant de rendre la main. 

  La fonction rend YaALireSur1__ (i.e. 1) si fd1 est seul lisible
                   YaALireSur_2_ (i.e. 2) si fd2 est seul lisible
		   YaALireSur12_ (i.e. 3) si fd1 et fd2 sont lisibles
		   YaALireSur__3 (i.e. 4) si fd3 est seul lisible
		   YaALireSur1_3 (i.e. 5) si fd1 et fd3 sont lisibles
		   YaALireSur_23 (i.e. 6) si fd2 et fd3 sont lisibles
		   YaALireSur123 (i.e. 7) si fd1, fd2 et fd3 sont lisibles
		   TIMEOUT       (i.e. 0) si aucun n'est lisible avant
		     une duree de timeout. 

  IMPORTANT : on travaille seulement en lecture.

\*================================================*/
{
    fd_set lecture, ecriture, except;
    int nbfds, nb_events, res; 
    struct timeval timer;

    /*---- Preparation du timer -------- */

    timer.tv_sec  = (int)floor(timeout); 
    timer.tv_usec = (int)floor(1000000*(timeout-(int)floor(timeout)));

    /*---- Preparation des ensembles -------- */
    /*--------- de descripteurs ------------- */
    
    nbfds = Max(fd1, fd2, fd3)+1; 
    if(nbfds<=0) nbfds=1; 
    
    FD_ZERO(&lecture); 
    FD_ZERO(&ecriture); 
    FD_ZERO(&except); 
    if(fd1>=0) FD_SET(fd1, &lecture); 
    if(fd2>=0) FD_SET(fd2, &lecture); 
    if(fd3>=0) FD_SET(fd3, &lecture); 
    
    /*------- attente d'un evenment --------- */
    
    if(timeout==0)
      nb_events = select (nbfds, &lecture, &ecriture, 
			  &except, NULL);
    else
      nb_events = select (nbfds, &lecture, &ecriture, 
			  &except, &timer);

    if(nb_events==-1) {perror("select"); return(CRASH);} 
    
    /*------- le depouillement des votes ---- */
    
    else
      {
	res = 0; 

	if((fd1>=0)&&(FD_ISSET(fd1, &lecture)))
	  res+=1; 

	if((fd2>=0)&&(FD_ISSET(fd2, &lecture)))
	  res+=2; 
	  
	if((fd3>=0)&&(FD_ISSET(fd3, &lecture)))
	  res+=4; 

	/* implicitement si aucune des trois conditions n'est */
	/* verifiee, alors c'est qu'on est en prensence d'un  */
	/* magnifique timeout : res = 0                       */
	
	return(res);
      }
}



