
/*===============================================*\
	Mardi 3 novembre 1998

	Arash HABIBI

	BitABitToolbox.c

	Quelques fonctions bien utiles. 

\*================================================*/


#include "BitABitToolbox.h"


/*----------- LES FONCTIONS DE BASE---------------*/
/*----------- POUR LE TYPE CHAR ------------------*/


int NiemeBitChar(char nombre, int n)
/*================================================*\
  Rend le n-ieme bit du nombre "nombre". 
  On commence avec le bit 0 pour les unites et on 
  finit avec le bit 7 pour 128, etc. 
\*================================================*/
{
  if((n<0)||(n>=8))
    {printf("NiemeBitChar : n doit etre entre 0 et 7 et non %d\n", n); 
    return(-1);}
  else
    return((nombre & (1<<n))>>n); 
}


void SetNiemeBitChar(char *nombre, int n, int value)
/*================================================*\
  Change le n-ieme bit pour le mettre a value
  (On commence avec le bit 1 pour les unites et 
  le bit 8 pour 128 etc. 
\*================================================*/
{
  if(value)
    *nombre = *nombre | (1<<n) ;
  else
    *nombre = *nombre & (~(1<<n)) ;
}


void ChangeNiemeBitChar(char *nombre, int n)
/*===============================================*\
  Change la valeur du n-ieme bit de car 0<->1.
\*===============================================*/
{
  *nombre = (*nombre)^(1<<n);
}


/*-------- LES FONCTIONS A UTILISER -------------*/
/*--------POUR TOUS LES AUTRES TYPES ------------*/


char NiemeBit(void *objet, int taille, int n)
/*=============================================*\
  taille est la taille de l'objet en octets. 
  On commence avec le bit 0 pour les unites. 
  Le bit 7 correspond au terme en x7; 
  Donc dans 19=10011 les bits 0, 1 et 4 sont a 1
  et les autres sont a 0. 
\*=============================================*/
{
  char *obj_loc; 

  if((n<0)||(n>=taille*8))
    {
      printf("Cet objet a %d octets donc n'a pas de bit numero %d.\n", 
	     taille, n);
      return(-1);
    }
  else
    {
      obj_loc=objet; 
      return(NiemeBitChar(obj_loc[taille-1-n/8], n%8));
    }
}


void SetNiemeBit(void *objet, int taille, int n, int value)
/*=============================================*\
  taille est la taille de l'objet en octets. 
  On commence avec le bit 0 pour les unites. 
  Le bit 7 correspond au terme en x7; 
  Donc dans 19=10011 les bits 0, 1 et 4 sont a 1
  et les autres sont a 0. 
  Cette fonction met le bit n a la valeur value. 
\*=============================================*/
{
  char *obj_loc; 

  if((n<0)||(n>=taille*8))
    {
      printf("Cet objet a %d octets donc n'a pas de bit numero %d.\n", 
	     taille, n);
    }
  else
    {
      obj_loc=objet; 
      SetNiemeBitChar(&(obj_loc[taille-1-n/8]), n%8, value);
    }
}


void ChangeNiemeBit(void *objet, int taille, int n)
/*=============================================*\
  taille est la taille de l'objet en octets. 
  On commence avec le bit 0 pour les unites. 
  Le bit 7 correspond au terme en x7; 
  Donc dans 19=10011 les bits 0, 1 et 4 sont a 1
  et les autres sont a 0. 
  Cette fonction change la valeur du bit 
  numero n : 1<->0.
\*=============================================*/
{
  char *obj_loc; 

  if((n<0)||(n>=taille*8))
    {
      printf("Cet objet a %d octets donc n'a pas de bit numero %d.\n", 
	     taille, n);
    }
  else
    {
      obj_loc=objet; 
      ChangeNiemeBitChar(&(obj_loc[taille-1-n/8]), n%8);
    }
}


void CheckBitABit(void *objet, int taille, char *commentaire)
/*===============================================*\
  Montre tous les bits d'un objet en mettant les 
  bits de poids faible a droite. 
\*===============================================*/
{
  int i,j; 

  printf("%s :", commentaire);  

  for(i=taille*8-1; i>=0; i--)
    {
      printf("%d", NiemeBit(objet, taille, i));
      if(i%8==0) printf(" ");
    }
  printf("\n");
}



int Degre(void *objet, int taille)
/*===============================================*\
  objet represente un polynome de degre au plus max. 
  Par exemple 10011. Degre donne le degre de ce 
  polynome (en l'occurrence 4)
\*===============================================*/
{
  int i;

  i=8*taille-1;

  while((i>=0)&&(NiemeBit(objet, taille, i)==0))
    i--; 
  
  if(i==-1) return(0);

  return(i); 
}


void TronquerANbits(void *objet, int taille, int N)
/*===============================================*\
  Annule les bits de poids N+1, N+2, ... qui 
  peuvent perturber les calculs quand il faut 
  regarder le degre des polynomes ou alors voir 
  si le reste est nul. 
  APRES L'APPEL DE CETTE FONCTION, L'OBJET SERA 
  DE DEGRE AU PLUS N. 
\*===============================================*/
{
  int i; 
  
  for(i=8*taille-1; i>N; i--)
    SetNiemeBit(objet, taille, i, 0);
}


void DecalageDroite(void *objet, int taille, int crans)
/*===============================================*\
  Quelque soit l'objet, decalse tous ces bits de 
  "crans" BITS vers la droite. (La droite signifie
  les bits de poids faible. (>>)
\*===============================================*/
{
  int i; 

  for(i=0; i<8*taille; i++)
    {
      if(i+crans<8*taille)
	SetNiemeBit(objet, 
		    taille, 
		    i, 
		    NiemeBit(objet, taille, i+crans));
      else
	SetNiemeBit(objet, taille, i, 0);
    }
}



