
/*===============================================*\
	Vendredi 18 Septembre 1998

	Arash HABIBI

	ServeurTCP.c

	Un espece de talk simplifie. 
	Ce cote-ci joue le role de serveur, 
	simplement dans le sens ou c'est lui 
	qui ecoute le reseau en attendant une
	demande de connexion. 
	Cette fois-ci, on est en mode bloquant
	et on utilise la primitive select. 

\*================================================*/


#include <sys/time.h>

#include "SocketToolbox.h"


void ConnexionServeur(short port_loc, int *sock_ecoute, int *sock_service)
/*===============================================*\
\*===============================================*/
{
	struct sockaddr_in adresse_loc, adresse_dist;
	int res, taille_adr, nb_pendants=1; 
	AdresseIP_t IP; 
	u_short port_dist;
	
/*-------- Creation et attachement de la socket d'ecoute ---------*/
	
	*sock_ecoute = GetAndBindSocket(SOCK_STREAM, 
		port_loc, &adresse_loc);
	
	printf("PapotServTCP attache au port numero %d\n",
		adresse_loc.sin_port);
		
		
/*-------- Debut de l'ecoute (Entree en service du serveur) ------*/
			
	res = listen(*sock_ecoute, nb_pendants);
	
	if(res == -1) {perror("listen"); exit(1);}
	else printf("PapotServTCP commence l'ecoute\n");
	
	
/*-------- La boucle de l'acceptation de la demande de cnxn -------*/

	taille_adr = sizeof(adresse_dist);

	*sock_service = accept(*sock_ecoute, 
		(struct sockaddr *)&adresse_dist, 
		&taille_adr);
	if(res==-1) {perror("accept");}
		
	/* --------connexion acceptee ------------*/

	ReadNetworkAddress(adresse_dist, &IP, &port_dist);
			
	printf("Demande de connexion venant de l'hote\
	     	\n%d.%d.%d.%d du port numero %d.\
	     	\nLa socket de service est le numero %d\n\n",
		IP.n1, IP.n2, IP.n3, IP.n4, port_dist, 
		*sock_service);
}



int Envoi(int terminal, int socket)
/*===============================================*\
  D'apres select nous savons qu'il y a 
  quelque chose a lire sur le terminal. 
  Le but de cette fonction est de lire
  ce quelque chose et de l'envoyer sur 
  la socket de service. 
  La fonction rend -1 en cas d'erreur 
  0 en cas de deconnexion et 1 si 
  tout va bien. 
\*===============================================*/
{
    int nb_lu, nb_sent;
    char message[100]; 
    
    nb_lu = read(terminal, message, 10);
    
    if(nb_lu<0) {perror("read terminal"); 
    	return(-1);}
    
    else if(nb_lu>0)
	{
	    nb_sent = write(socket, message, nb_lu);
	    
	    if(nb_sent<0) {perror("write socket"); 
	    	return(-1);}
	    
	    else if(nb_sent==0) /* socket distante fermee */
		return(-1); /* sans crier gare ------ */
	    
	    else if(nb_sent>0) 
		if((nb_lu==2)&&
		   (strncmp(message, ".", nb_lu-1)==0))
		    return(0); 
		else
		    /* rien a faire tout va bien*/
		    return(1);
	}
}


int Reception(int terminal, int socket)
/*===============================================*\
  D'apres select nous savons qu'il y a 
  quelque chose a lire sur la socket. 
  Le but de cette fonction est de lire
  ce quelque chose et de l'envoyer sur 
  le terminal de lecture. 
  La fonction rend -1 en cas d'erreur 
  0 en cas de deconnexion et 1 si 
  tout va bien. 
\*===============================================*/
{
    int nb_ecrit, nb_rec;
    char message[100]; 
    
    nb_rec = read(socket, message, 10);
    
    if(nb_rec==-1) {perror("read socket"); 
    return(-1);}
    
    else if(nb_rec==0) /* socket distante fermee */
	return(-1); 	   /* sans crier gare ------ */
    
    else if(nb_rec>0)
	{
	    if((nb_rec==2)&&
	       (strncmp(message, ".", nb_rec-1)==0))
		return(0); 	
	    else
		{
		    nb_ecrit = write(terminal, 
				     message, nb_rec);
		    
		    if(nb_ecrit==-1) 
			{perror("write terminal"); 
			return(-1);}
		    
		    else if(nb_ecrit>0)
			/* rien a faire tout va bien*/
			return(1);
		    
		    else
			printf("Probleme : nb_ecrit = %d\n", 
			       nb_ecrit); return(-1);  
		}
	}
    else
	{printf("Probleme : nb_rec = %d\n", nb_rec);
	return(-1);}
}


void main(int argc, char **argv)
/*===============================================*\
\*===============================================*/
{
	int term_rec, term_send, sock_ecoute, sock_service; 
	int transmission_over, crash;
	int nbfds, nb_events, nb_lu, nb_recu, res; 
	fd_set lecture, ecriture, except;
	 
	if(argc!=2) 
	{printf("Usage : PapotServ <NomTerminal>\n"); exit(1);}

/*-------- Ouvertures diverses -------- ------------------*/

	/*-- Le terminal sur lequel on enverra -------*/

	term_send = open("/dev/tty", O_RDWR); 
	if(term_send == -1) 
	    {perror("terminal de controle"); exit(1);}

	/*-- Le terminal sur lequel on recevra -------*/

	term_rec = open(argv[1], O_RDWR); 
	if(term_rec == -1) 
	    {perror("terminal de lecture"); exit(1);}

/*-------- Connexion ------------------ ------------------*/
	
	ConnexionServeur(0, &sock_ecoute, &sock_service);
	transmission_over = 0; crash = 0; 
	      
/*-------- Elements permanents de select ----------------*/

	if(term_send>sock_service) nbfds = term_send+1; 
		else nbfds = sock_service+1;

/*-------- La boucle de traitement des evenements --------*/

	while((!crash)&&(!transmission_over))
	{
		/*---- Preparation des ensembles -------- */
		/*--------- de descripteurs ------------- */
		
		FD_ZERO(&lecture); 
		FD_ZERO(&ecriture); 
		FD_ZERO(&except); 
		FD_SET(term_send, &lecture); 
		FD_SET(sock_service, &lecture); 
		
		/*------- attente d'un evenment --------- */
		
		nb_events = select (nbfds, &lecture, &ecriture, 
			&except, NULL);
			
		if(nb_events==-1) {perror("select"); crash=1;} 
		
		/*------- le depouillement des votes ---- */
		
		if(FD_ISSET(term_send, &lecture))
		{
			res = Envoi(term_send, sock_service);
			
			if(res==-1) crash=1; 
			else if(res==0) transmission_over=1; 
			else if(res>0) /* tout va bien */;
		}
				
		if(FD_ISSET(sock_service, &lecture))
		{	
			res = Reception(term_rec, sock_service);
			
			if(res==-1) crash=1; 
			else if(res==0) transmission_over=1; 
			else if(res>0) /* tout va bien */;
		}
	}

	close(sock_service);
	close(sock_ecoute);
	if(crash)
		printf("Echange interrompu\n\n");
	else if(transmission_over)
		printf("Echange termine. Deconnexion.\n\n");
}
